@@ -5,7 +5,7 @@
  * Author: Giovanni Giacobbi <johnny@themnemonic.org>
  * Copyright (C) 2002  Giovanni Giacobbi
  *
- * $Id: network.c,v 1.16 2002/05/08 20:29:37 themnemonic Exp $
+ * $Id: network.c,v 1.20 2002/05/15 20:20:00 themnemonic Exp $
  */
 
 /***************************************************************************
@@ -93,8 +93,7 @@ bool netcat_resolvehost(netcat_host *dst, char *name)
   }
   else {			/* `name' is a numeric address, try reverse lookup */
     memcpy(&dst->iaddrs[0], &res_addr, sizeof(dst->iaddrs[0]));
-    /* FIXME: the following is broken cause of dst->addrs */
-    strncpy(dst->addrs[0], netcat_inet_ntop(&res_addr), sizeof(dst->addrs));
+    strncpy(dst->addrs[0], netcat_inet_ntop(&res_addr), sizeof(dst->addrs[0]));
 
     /* if opt_numeric is set or we don't require verbosity, we are done */
     if (opt_numeric || !opt_verbose)
@@ -235,28 +234,35 @@ const char *netcat_inet_ntop(const void *src)
   return ret;
 }			/* end of netcat_inet_ntop() */
 
-/* ... */
+/* Backend for the socket(2) system call.  This function wraps the creation of
+   new sockets and sets the common SO_REUSEADDR SOL_SOCKET option, handling
+   eventual errors.
+   Returns -1 if the socket(2) call failed, -2 if the setsockopt() call failed;
+   otherwise the return value is a descriptor referencing the new socket. */
 
-int netcat_socket_new()
+int netcat_socket_new(int domain, int type)
 {
   int sock, ret, sockopt = 0;
 
-  sock = socket(PF_INET, SOCK_STREAM, 0);
+  sock = socket(domain, type, 0);
   if (sock < 0)
     return -1;
 
   /* fix the socket options */
   ret = setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &sockopt, sizeof(sockopt));
-  if (ret < 0)
+  if (ret < 0) {
+    close(sock);		/* anyway the socket was created */
     return -2;
+  }
 
   return sock;
 }
 
 /* ... */
 
-int netcat_socket_new_connect(const struct in_addr *addr, unsigned short port,
-		const struct in_addr *local_addr, unsigned short local_port)
+int netcat_socket_new_connect(int domain, int type, const struct in_addr *addr,
+		unsigned short port, const struct in_addr *local_addr,
+		unsigned short local_port)
 {
   int sock, ret;
   struct sockaddr_in rem_addr;
@@ -269,9 +275,9 @@ int netcat_socket_new_connect(const struct in_addr *addr, unsigned short port,
   memcpy(&rem_addr.sin_addr, addr, sizeof(rem_addr.sin_addr));
 
   /* create the socket and fix the options */
-  sock = netcat_socket_new();
+  sock = netcat_socket_new(domain, type);
   if (sock < 0)
-    return sock;
+    return sock;		/* just forward the error code */
 
   /* only if needed, bind it to a local address */
   if (local_addr || local_port) {
@@ -286,23 +292,44 @@ int netcat_socket_new_connect(const struct in_addr *addr, unsigned short port,
     else
       memset(&my_addr.sin_addr, 0, sizeof(my_addr.sin_addr));
     ret = bind(sock, (struct sockaddr *)&my_addr, sizeof(my_addr));
-    if (ret < 0)
-      return -3;
+    if (ret < 0) {
+      ret = -3;
+      goto err;
+    }
   }
 
   /* add the non-blocking flag to this socket */
-  ret = fcntl(sock, F_SETFL, fcntl(sock, F_GETFL, 0) | O_NONBLOCK);
-  if (ret < 0)
-    return -4;
+  if ((ret = fcntl(sock, F_GETFL, 0)) >= 0)
+    ret = fcntl(sock, F_SETFL, ret | O_NONBLOCK);
+  if (ret < 0) {
+    ret = -4;
+    goto err;
+  }
 
   /* now launch the real connection. Since we are in non-blocking mode, this
      call will return -1 in MOST cases (on some systems, a connect() to a local
      address may immediately return successfully) */
   ret = connect(sock, (struct sockaddr *)&rem_addr, sizeof(rem_addr));
-  if ((ret < 0) && (errno != EINPROGRESS))
-    return -5;
+  if ((ret < 0) && (errno != EINPROGRESS)) {
+    ret = -5;
+    goto err;
+  }
 
+  /* everything went fine, return the (maybe connecting) socket */
   return sock;
+
+ err:
+  if (ret < 0) {
+    int tmpret, saved_errno = errno;
+
+    /* the close() call MUST NOT fail */
+    tmpret = close(sock);
+    assert(tmpret >= 0);
+
+    /* restore the original errno */
+    errno = saved_errno;
+  }
+  return ret;
 }
 
 /* ... */
@@ -320,26 +347,45 @@ int netcat_socket_new_listen(const struct in_addr *addr, unsigned short port)
   memcpy(&my_addr.sin_addr, addr, sizeof(my_addr.sin_addr));
 
   /* create the socket and fix the options */
-  sock = netcat_socket_new();
+  sock = netcat_socket_new(PF_INET, SOCK_STREAM);
   if (sock < 0)
-    return sock;
+    return sock;		/* just forward the error code */
 
   /* bind it to the specified address (could be INADDY_ANY as well) */
   ret = bind(sock, (struct sockaddr *)&my_addr, sizeof(my_addr));
-  if (ret < 0)
-    return -3;
+  if (ret < 0) {
+    ret = -3;
+    goto err;
+  }
 
   /* actually make it listening, with a reasonable backlog value */
   ret = listen(sock, 4);
-  if (ret < 0)
-    return -4;
+  if (ret < 0) {
+    ret = -4;
+    goto err;
+  }
 
   return sock;
+
+ err:
+  if (ret < 0) {
+    int tmpret, saved_errno = errno;
+
+    /* the close() call MUST NOT fail */
+    tmpret = close(sock);
+    assert(tmpret >= 0);
+
+    /* restore the original errno */
+    errno = saved_errno;
+  }
+  return ret;
 }
 
 /* This function is much like the accept(2) call, but implements also the
-   parameter `timeout', which specifies the time (in seconds) after which
-   the function returns.
+   parameter `timeout', which specifies the time (in seconds) after which the
+   function returns.  If `timeout' is negative, the remaining of the last
+   valid timeout specified is used.  If it reached zero, or if the timeout
+   haven't been initialized already, this function will wait forever.
    Returns -1 on error, setting the errno variable.  If it succeeds, it
    returns a non-negative integer that is the descriptor for the accepted
    socket. */
@@ -347,18 +393,26 @@ int netcat_socket_new_listen(const struct in_addr *addr, unsigned short port)
 int netcat_socket_accept(int s, int timeout)
 {
   fd_set in;
-  struct timeval timest;
+  static bool timeout_init = FALSE;
+  static struct timeval timest;
 
   debug_v("netcat_socket_accept(s=%d, timeout=%d)", s, timeout);
 
   /* initialize the select() variables */
   FD_ZERO(&in);
   FD_SET(s, &in);
-  timest.tv_sec = timeout;
-  timest.tv_usec = 0;
+  if (timeout > 0) {
+    timest.tv_sec = timeout;
+    timest.tv_usec = 0;
+    timeout_init = TRUE;
+  }
+  else if (timeout && !timeout_init) {
+    /* means that timeout is < 0 and timest hasn't been initialized */
+    timeout = 0;
+  }
 
-  /* now go into select. use the timeout only if it is valid */
-  select(s + 1, &in, NULL, NULL, (timeout > 0 ? &timest : NULL));
+  /* now go into select. use timest only if we don't wait forever */
+  select(s + 1, &in, NULL, NULL, (timeout ? &timest : NULL));
 
   /* have we got this connection? */
   if (FD_ISSET(s, &in)) {
@@ -372,6 +426,10 @@ int netcat_socket_accept(int s, int timeout)
     return new_sock;
   }
 
+  /* since we've got a timeout, the timest is now zero and thus it is like
+     uninitialized.  Next time assume wait forever. */
+  timeout_init = FALSE;
+
   /* no connections arrived during the given time. nothing happens */
   errno = ETIMEDOUT;
   return -1;
